// BufferedStreamSampleCP.cpp : main project file.
// <summary>
// This sample demonstrates how to use the BufferedStream class to
// improve IO performance on a network stream. This sample is a
// revision of the network stream Sender sample that includes the
// composable stream BufferedStream.
// 
// sample has 2 optional parameters that can be used
// 
// Sender [/server <server IP>] [/port <port>]
// By default this application will connect to the server on the same machine
// using TCP port 5150.
// 
// </summary>

#include "stdafx.h"

using namespace System;
using namespace System::IO;
using namespace System::Net;
using namespace System::Net::Sockets;

// <summary>
// The main entry point for the application.
// </summary>
[STAThread]
int main(array<System::String ^> ^args)
{
    // The folowing are the default values
    String^ ServerName = "127.0.0.1";
    int Port = 5150;

	args = Environment::GetCommandLineArgs();
	
	if(args->Length != 1)
	{
		// Parse command line arguments if any
		for (int i = 0; i < args->Length; i++)
		{
			if (String::Compare(args[i], "/server", true) == 0)
			{
                // The server's name we will connect to
                ServerName = args[++i]->ToString();
            }
			else if (String::Compare(args[i], "/port", true) == 0)
            {
                // The port on which the server is listening
				Port = System::Convert::ToInt32(args[++i]->ToString());
            }
        }
	}
	else
	{
		Console::WriteLine("Usage: {0} [/server <server IP>] [/port <port>]", args[0]);
		return 0;
	}

    Socket^ ClientSocket = nullptr;

    try
    {
        // Let's connect to a listening server
		Console::WriteLine("Connecting to listening server...");
        try
        {
			ClientSocket = gcnew Socket(AddressFamily::InterNetwork, SocketType::Stream, ProtocolType::IP);
			Console::WriteLine("Socket() is OK...");
        }
        catch (Exception^ e)
        {
            throw gcnew Exception("Failed to create client Socket: " + e->Message);
        }

		IPEndPoint^ ServerEndPoint = gcnew IPEndPoint(IPAddress::Parse(ServerName), Convert::ToInt16(Port));

        try
        {
			ClientSocket->Connect(ServerEndPoint);
			Console::WriteLine("Connect() is OK...");
        }
        catch (Exception^ e)
        {
            throw gcnew Exception("Failed to connect client Socket: " + e->Message);
        }
    }
    catch (Exception^ e)
    {
		Console::WriteLine(e->Message);
		Console::WriteLine("Closing Socket...");
        ClientSocket->Close();
        return 0;
    }

    // Let's create a network stream to communicate over the connected Socket.
    NetworkStream^ ClientNetworkStream = nullptr;

    try
    {
        try
        {
            // Setup a network stream on the client Socket
			Console::WriteLine("Instantiate NetworkStream object...");
            ClientNetworkStream = gcnew NetworkStream(ClientSocket, true);
        }
        catch (Exception^ e)
        {
            // We have to close the client socket here because the network
            // stream did not take ownership of the socket.
            throw gcnew Exception("Failed to create a NetworkStream with error: " + e->Message);
			Console::WriteLine("Closing client Socket...");
            ClientSocket->Close();
        }

    }
    catch (Exception^ e)
    {
		Console::WriteLine(e->Message);
		Console::WriteLine("Closing client NetworkStream...");
        ClientNetworkStream->Close();
        return 0;
    }

    BufferedStream^ ClientBufferedStream = nullptr;

    try
    {
        try
        {
            // Setup a network stream on the client Socket
			Console::WriteLine("Instantiate BufferedStream object...");
            ClientBufferedStream = gcnew BufferedStream(ClientNetworkStream);
        }
        catch (Exception^ e)
        {
            throw gcnew Exception("Failed to create a BufferedStream with error: " + e->Message);
        }

        try
        {
			Console::WriteLine("Writing/sending data...");
			array< Byte >^ Buffer = gcnew array< Byte >(1);

			for (int i = 0; i < 200; i++)
            {
                Buffer[0] = (Byte)i;
				ClientBufferedStream->Write(Buffer, 0, Buffer->Length);
            }

			Console::WriteLine("We wrote 200 bytes one byte at a time to the server.");
        }
        catch (Exception^ e)
        {
            throw gcnew Exception("Failed to write to client BufferedStream with error: " + e->Message);
        }
    }
    catch (Exception^ e)
    {
		Console::WriteLine(e->Message);
    }
    finally
    {
        // We are finished with the Stream so we will close it.
		Console::WriteLine("Closing the BufferedStream...");
        ClientBufferedStream->Close();
    }
return 0;
}
